{$N+,G+}
uses
  Graph,
  TPString,
  TPCrt,
  Gradient,
  WriteBMP;

type
  RGB=record
    R,G,B:byte;
  end;

const
  MaxX=799;
  MaxY=599;

  Pmin:extended=-2.75;
  Pmax:extended=1.25;
  Qmin:extended=-1.5;

  {Pmin:extended=-0.7454356;
  Pmax:extended=-0.7454215;
  Qmin:extended=0.1130076;}

  MaxIter:word=255;
  M:word=100;
  PalNumber:byte=2;

  FileNum: integer=0;

var
  RectWidth, RectHeight, RectX0, RectY0:integer;

procedure GenPal(Num:byte);
begin
  case Num of
    1:GenPalette(Grad1);
    2:GenPalette(Grad2);
    3:GenPalette(Grad3);
    4:GenPalette(Grad4);
    5:GenPalette(Grad5);
  end;
end;

procedure DrawMandelbrot(Pmin ,Pmax, Qmin:extended; MaxIter, M:word);

var
  X,Y,Color,Size:word;
  Delta, P, Q, X2, Y2, NewX2, ColorConstant: extended;
  MsgStr:string;

begin
  ColorConstant:=MaxIter/255;
  Delta:=(Pmax-Pmin)/MaxX;

  for Y:=0 to MaxY do
  for X:=0 to MaxX do
  begin
    P:=Pmin+X*Delta;
    Q:=Qmin+Y*Delta;
    Color:=0;
    X2:=0;
    Y2:=0;
    repeat
      NewX2:=X2*X2-Y2*Y2+P;
      Y2:=2*X2*Y2+Q;
      X2:=NewX2;
      Inc(Color);
      if Sqrt(X2*X2 + Y2*Y2)>M then
      begin
        PutPixel(X, MaxY-Y, Round(Color/ColorConstant) );
        Break;
      end;
      if Color=MaxIter then
      begin
        PutPixel(X, MaxY-Y, 0);
        Break;
      end;
    until false;
  end;
  MsgStr:= '  Re C : '+Real2Str(Pmin, 11, 8)+' ... '+Real2Str(Pmax, 11, 8)+
          ' ; Im C : '+Real2Str(Qmin, 11, 8)+' ... '+Real2Str(Qmin+Delta*MaxY, 11, 8);

  SetColor(0);
  OutTextXY(0, 8, MsgStr);
end;

procedure RectOnOff;
begin
  SetwriteMode(XORPut);
  SetColor(255);
  Rectangle(RectX0, RectY0, RectX0+RectWidth, RectY0+ RectHeight);
end;

procedure RectRight;
begin
  if (RectX0+RectWidth)>=MaxX then Exit;
  RectOnOff;
  Inc(RectX0);
  RectOnOff;
end;

procedure RectLeft;
begin
  if RectX0<=0 then Exit;
  RectOnOff;
  Dec(RectX0);
  RectOnOff;
end;

procedure RectUp;
begin
  if RectY0<=0 then Exit;
  RectOnOff;
  Dec(RectY0);
  RectOnOff;
end;

procedure RectDown;
begin
  if (RectY0+RectHeight)>=MaxY then Exit;
  RectOnOff;
  Inc(RectY0);
  RectOnOff;
end;

procedure FastRectRight;
begin
  if (RectX0+RectWidth+10)>=MaxX then Exit;
  RectOnOff;
  Inc(RectX0,10);
  RectOnOff;
end;

procedure FastRectLeft;
begin
  if RectX0<=10 then Exit;
  RectOnOff;
  Dec(RectX0,10);
  RectOnOff;
end;

procedure FastRectUp;
begin
  if RectY0<=10 then Exit;
  RectOnOff;
  Dec(RectY0,10);
  RectOnOff;
end;

procedure FastRectDown;
begin
  if (RectY0+RectHeight+10)>=MaxY then Exit;
  RectOnOff;
  Inc(RectY0,10);
  RectOnOff;
end;

procedure RectPlus;
begin
  RectOnOff;
  Inc(RectWidth);
  RectHeight:=Round(3*RectWidth/4);
  RectOnOff;
end;

procedure RectMinus;
begin
  if (RectWidth<=1) or (RectHeight<=1) then Exit;
  RectOnOff;
  Dec(RectWidth);
  RectHeight:=Round(3*RectWidth/4);
  RectOnOff;
end;

var
  Drv, Mode:integer;
  Key:word;
  Delta:extended;

begin
  Drv := InstallUserDriver('SVGA256',nil);
  Mode:=3; {800x600x256}
  InitGraph(Drv, Mode, '');
  GenPal(PalNumber);
  DrawMandelbrot(Pmin, Pmax, Qmin, MaxIter, M);
  {DrawMandelbrot(-0.7454356, -0.7454215, 0.1130076);}
  {DrawMandelbrot(-1.781, -1.782, 0);}
  RectWidth:=40;
  RectHeight:=30;
  RectX0:=0;
  RectY0:=0;
  RectOnOff;
  repeat
    Key:=ReadKeyWord;
    case Key of
      $0231: {'1'}
      begin
        PalNumber:=1;
        GenPal(PalNumber);
      end;
      $0332: {'2'}
      begin
        PalNumber:=2;
        GenPal(PalNumber);
      end;
      $0433: {'3'}
      begin
        PalNumber:=3;
        GenPal(PalNumber);
      end;
      $0534: {'4'}
      begin
        PalNumber:=4;
        GenPal(PalNumber);
      end;
      $0635: {'5'}
      begin
        PalNumber:=5;
        GenPal(PalNumber);
      end;

      $4B00: {Left}  RectLeft;
      $4D00: {Right} RectRight;
      $4800: {Up}    RectUp;
      $5000: {Down}  RectDown;

      $4700: {Home}  FastRectUp;
      $4F00: {End}   FastRectDown;
      $5300: {Del}   FastRectLeft;
      $5100: {PgDn}  FastRectRight;

      $4E2B: {Gray+} RectPlus;
      $4A2D: {Gray-} RectMinus;

      $1C0D: {Enter}
        begin
          Delta:=(Pmax-Pmin)/MaxX;
          Pmin:=Pmin+RectX0*Delta;
          Pmax:=Pmin+Delta*RectWidth;
          Qmin:=Qmin+(MaxY-RectY0-RectHeight)*Delta;
          RectOnOff;
          DrawMandelbrot(Pmin, Pmax, Qmin, MaxIter, M);
          RectOnOff;
        end;
      $1E00: {Alt+A}
        begin
          Pmin:=-2.75;
          Pmax:=1.25;
          Qmin:=-1.5;
          RectOnOff;
          DrawMandelbrot(Pmin, Pmax, Qmin, MaxIter, M);
          RectOnOff;
        end;
      $1700: {Alt+I}
        begin
          RectOnOff;
          RestoreCrtMode;
          Write('Enter MaxIter [', MaxIter, ']:');
          Readln(MaxIter);
          SetGraphMode(3);
          GenPal(PalNumber);
          DrawMandelbrot(Pmin, Pmax, Qmin, MaxIter, M);
          RectOnOff;
        end;
      $3200: {Alt+M}
        begin
          RectOnOff;
          RestoreCrtMode;
          Write('Enter MaxLimit [', M, ']:');
          Readln(M);
          SetGraphMode(3);
          GenPal(PalNumber);
          DrawMandelbrot(Pmin, Pmax, Qmin, MaxIter, M);
          RectOnOff;
        end;
      $2100: {Alt+F}
        begin
          RectOnOff;
          WriteBMPFile(0, 0, 799, 599, 'scr_'+Long2Str(FileNum)+'.bmp');
          RectOnOff;
          Inc(FileNum);
        end;
    end;
  until Key=$011B;
  CloseGraph;
end.
